"""
Read the data from xml files given in the config.py file.

Sum the time intervals and the x, y, z coordnates of displacements for
those time instants when the displacement dr = sqrt(x**2 + y**2 + z**2)
is nonzero.

Calculate the total distance from these sums for every swimmer. Thus, there
will be one total distance value per swimmer.

Plot this distance distribution as a histogram plot. Thus this analysis takes
into account the total distance of the swimming as a difference between the
end and starting points.
"""

import numpy as np
import pylab as py
import swim_config as cf
import swimlib_xml as sx
import swimlib as s


# generate path for output folder
folder_out = s.results_folder(cf.path_in)

nrows = len(cf.files)
ncols = 1
fig, axs = py.subplots(nrows=nrows, ncols=ncols, figsize=(10, 13))

for data_index, file_name in enumerate(cf.files):

    path = cf.path_in + '/' + file_name
    tracks = sx.extract_tracks(path)
    num_particles = sx.extract_num_particles(tracks)
    frame_interval = sx.extract_frame_interval(tracks)

    print(f"{file_name}")
    print(f"{num_particles} particles\n")

    R_array = []
    for particle in tracks:

        t, x, y, z = sx.extract_coordinates(particle)

        t *= frame_interval

        dt = np.diff(t)
        dx = np.diff(x)
        dy = np.diff(y)
        dz = np.diff(z)
        dr = np.sqrt(dx**2 + dy**2 + dz**2)

        T = sum(dt[dr > cf.dist_thr])  # only when the swimmer moves
        X = sum(dx[dr > cf.dist_thr])
        Y = sum(dy[dr > cf.dist_thr])
        Z = sum(dz[dr > cf.dist_thr])

        # Note that it is essential to calculate distance NOT based on dr,
        # because this doesn't take into account the swimmers going back and
        # forth.
        R = np.sqrt(X**2.0 + Y**2.0 + Z**2.0)

        R_array = np.r_[R_array, R]

    axs[data_index].hist(R_array, 20,
                         histtype='stepfilled',
                         facecolor='g', alpha=0.75)
    axs[data_index].set_title(file_name[:-4])
    axs[data_index].set_ylabel('$count$', fontsize=12)

    np.savetxt(folder_out + '/' + file_name[:-4] + '_total_distance_histogram.txt',
               R_array,
               fmt='%10.5f')

axs[nrows-1].set_xlabel('$swimmed$ $distance$ $[\mu m]$', fontsize=12)

py.tight_layout()
py.savefig(folder_out + '/swim_total_distance_histogram.pdf')
# py.show()
