"""
Read the data from xml files given in the config.py file.

Sum the time intervals and the x, y, z coordnates of displacements for those
time instants when the displacement dr = sqrt(x**2 + y**2 + z**2) is nonzero.

Calculate the velocity as the ratio of the total distance and total time
determined in this way of every swimmer. Thus, there will be one velocity value
per swimmer.

Plot this velocity distribution as a box plot. This analysis takes into account
the total distance of the swimming as a difference between the end and starting
points. It also take into account the time when the swimmer doesn't swim.
"""

import numpy as np
import pylab as py
import swim_config as cf
import swimlib_xml as sx
import swimlib as s


# generate path for output folder
folder_out = s.results_folder(cf.path_in)

fig, axs = py.subplots(nrows=1, ncols=1, figsize=(6, 4))

for data_index, file_name in enumerate(cf.files):

    path = cf.path_in + '/' + file_name
    tracks = sx.extract_tracks(path)
    num_particles = sx.extract_num_particles(tracks)
    frame_interval = sx.extract_frame_interval(tracks)

    print(f"{file_name}")
    print(f"{num_particles} particles\n")

    V_array = []
    for particle in tracks:

        t, x, y, z = sx.extract_coordinates(particle)

        t *= frame_interval

        dt = np.diff(t)
        dx = np.diff(x)
        dy = np.diff(y)
        dz = np.diff(z)
        dr = np.sqrt(dx**2 + dy**2 + dz**2)

        T = sum(dt[dr > cf.dist_thr])  # only when the swimmer moves
        X = sum(dx[dr > cf.dist_thr])
        Y = sum(dy[dr > cf.dist_thr])
        Z = sum(dz[dr > cf.dist_thr])

        if T > 0:
            V = np.sqrt(X**2.0 + Y**2.0 + Z**2.0) / T
            V_array = np.r_[V_array, V]

    axs.boxplot(V_array, positions = [data_index,],
                notch=False,
                showmeans=True,
                showfliers=False)

    np.savetxt(folder_out + '/' + file_name[:-4] + '_velocity_from_total_distance.txt',
               V_array,
               fmt='%10.5f')

axs.set_ylabel('$v$ $[\mu m\,s^{-1}]$', fontsize=16)

py.tight_layout()
py.savefig(folder_out + '/swim_velocity_from_total_distance.pdf')

# py.show()
