"""
This script only plots the velocity calculation based on segmenting the raw
data.

Plotted in each subfigure title is:

- file index
- (mean velocity, max velocity)

We also calculate and print out the mean travelled distance. Note that it is
important to base the calculation on summing the coordinate changes dx, dy, dz
and not the total increment, because the swimmers fluctuate back and forth,
leading to travelled distance cancelation not reflected in dr.
"""

import numpy as np
import pylab as py
import swim_config as cf
import swimlib_xml as sx
import swimlib as s


# generate path for output folder
folder_out = s.results_folder(cf.path_in)

np.random.seed(42)

nrows = 7
ncols = 10

for data_index, file_name in enumerate(cf.files):

    path = cf.path_in + '/' + file_name
    tracks = sx.extract_tracks(path)
    num_particles = sx.extract_num_particles(tracks)
    frame_interval = sx.extract_frame_interval(tracks)
    particle_list = sx.extract_particle_list(tracks)

    print(f'{file_name}')
    print(f'{num_particles} particles')

    # get the maximum velocity for plot normalisation.
    maxv_all = s.velocity_max(num_particles,
                              particle_list,
                              frame_interval) 

    fig, axs = py.subplots(nrows=nrows, ncols=ncols, figsize=(13, 7),
                           subplot_kw={'xticks': [], 'yticks': []})
    fig.suptitle(f'Dataset "{file_name}" (max axis velocity $v_m$ ={str(maxv_all)[:5]} $\mu$m/s)')


    index_list = list(range(num_particles)) # for random particle generation
    mean_R = []
    for i in range(nrows):
        for j in range(ncols): # because last file is always empty

            # for random particle generation
            if index_list == []: # quick fix if num particles < num images to plot
                index_list = [0]
            index = np.random.choice(index_list)
            index_list.remove(index)

            t, v, dx, dy, dz, dr, x, y, z = \
                s.velocity_instantaneous(particle_list,
                                         index,
                                         frame_interval)

            # plot cumulative distance as visual guide (arbitrary units)
            dist_cumul = 0.5*maxv_all*np.cumsum(dr)/max(np.cumsum(dr))
            dist_cumul_fit = s.fit_wedge(t[:-1], dist_cumul)[0]

            v_acorr = s.autocorr_np(v) + maxv_all/2.0

            axs[i][j].plot(t[:-1], v, 'r', lw=0.5)
            axs[i][j].plot(t[:-1], v_acorr, 'g', lw=0.5)
            axs[i][j].plot(t[:-1], dist_cumul , 'b', lw=0.5)
            axs[i][j].plot(t[:-1], dist_cumul_fit , 'b--', lw=0.5)
            axs[i][j].plot(t, np.zeros(len(t)), 'k', lw=0.5)
            axs[i][j].set_title(str(index) + " (" +
                                str(np.mean(v))[:4] + "," +
                                str(np.max(v))[:4] + ")",
                                fontsize=10)
            axs[i][j].set_ylim([-1, maxv_all])
            R = np.sqrt(sum(dx)**2 + sum(dy)**2 + sum(dz)**2)
            mean_R.append(R)

    print(f'Mean travelled distance: {np.mean(mean_R)}\n')

    py.tight_layout()
    py.savefig(folder_out + '/' + file_name[:-4] + '_velocity_instantaneous_fluct.pdf')

# py.show()
