import os
import numpy as np
import swimlib_xml as sx
from scipy.optimize import least_squares
from sklearn.metrics import r2_score


def results_folder(path_in):
    """
    Create path to 'results' folder. Generate copies of all subfolders as
    given in the input path. Check if the individual folders exists, if
    not create them.
    """
    folder_out = "results"

    if not os.path.isdir(folder_out):
        os.mkdir(folder_out)

    folders = path_in.split("/")
    
    for f in folders[1:]:
        folder_out = os.path.join(folder_out, f)
        if not os.path.isdir(folder_out):
            os.mkdir(folder_out)

    return folder_out


def velocity_instantaneous(particle_list, index, frame_interval):
    """
    Return an array of instantaneous velocity and the 
    corresponding time array.
    """

    t, x, y, z = sx.extract_coordinates(particle_list[index])

    t *= frame_interval

    dt = np.diff(t)
    dx = np.diff(x)
    dy = np.diff(y)
    dz = np.diff(z)
    dr = np.sqrt(dx**2 + dy**2 + dz**2)
    v = dr/dt

    return t, v, dx, dy, dz, dr, x, y, z


def velocity_max(num_particles, particle_list, frame_interval):
    """
    Determine maximum velocity for plot axis normalisation.
    """
    maxv = -1.0
    for i in range(num_particles):

        o = velocity_instantaneous(particle_list,
                                   i,
                                   frame_interval)
        v = o[1]

        if maxv < max(v):
            maxv = max(v)

    return maxv


def autocorr_np(data):
    """
    Autocorrelation function calculation using Numpy.
    
    This is from here:
    https://scicoding.com/4-ways-of-calculating-autocorrelation-in-python/
    
    Much faster than autocorr.
    """
    
    mean = np.mean(data)
    var = np.var(data)
    ndata = data - mean

    acorr = np.correlate(ndata, ndata, 'full')[len(ndata)-1:] 
    acorr = acorr / var / len(ndata)
    
    return acorr


def fit_poly(x, y, n=4):
    """
    fit polynomial to the data
    """

    def fun(p, x, n):
        y_fun = 0
        for i in range(n):
            y_fun += p[i] * x**i
        return y_fun

    def errorfun(p, x, y, n):
        return y - fun(p, x, n)

    p = np.ones(n) 
    res = least_squares(errorfun, p, args=(x, y, n)) 

    r2 = r2_score(y, fun(res.x, x, n))

    return fun(res.x, x, n), res.x, r2


def fit_wedge(x, y):
    """
    fit piece-wise linear function to the data.
    """

    xa, ya = x[0], y[0]
    xb, yb = x[-1], y[-1]

    vel_av = (yb-ya) / (xb-xa)
    time_tot = xb-xa
    dist_tot = yb-ya

    err_min = 1e10
    for xf, yf in zip(x, y):

        if xf == xa:
            # line with origin in the point xb (because y1[x<=xf] = [])
            y_fit = ((yb-yf)*x + (yf*xb-yb*xf)) / (xb-xf)
            vel =  [np.nan, (yb-yf) / (xb-xf), vel_av]
            time = [np.nan, (xb-xf), time_tot]
            dist = [np.nan, (yb-yf), dist_tot]
        elif xf == xb:
            # line with origin in the point xa (because y2[x>xf] = [])
            y_fit = ((yf-ya)*x + (ya*xf-yf*xa)) / (xf-xa)
            vel =  [(yf-ya) / (xf-xa), np.nan, vel_av]
            time = [(xf-xa), np.nan, time_tot]
            dist = [(yf-ya), np.nan, dist_tot]
        else:
            y1 = ((yf-ya)*x + (ya*xf-yf*xa)) / (xf-xa)
            y2 = ((yb-yf)*x + (yf*xb-yb*xf)) / (xb-xf)
            y_fit = np.r_[y1[x<=xf], y2[x>xf]]
            vel = [(yf-ya) / (xf-xa), (yb-yf) / (xb-xf), vel_av]
            time = [xf-xa, xb-xf, time_tot]
            dist = [yf-ya, yb-yf, dist_tot]


        err = sum((y-y_fit)**2)
        r2 = r2_score(y, y_fit)

        if err<err_min:
            err_min = err
            y_best = np.copy(y_fit)
            vel_best = np.copy(vel)
            time_best = np.copy(time)
            dist_best = np.copy(dist)
            r2_best = r2

    return y_best, vel_best, time_best, dist_best, r2_best

